#!/usr/bin/env python3

# This amulet test deploys the bundles.yaml file in this directory.

import os
import unittest
import yaml
import amulet
import sys
import requests

# Lots of prereqs on this charm (eg: java), so give it a large timeout
seconds_to_wait = 2400


class BundleTest(unittest.TestCase):
    """ Create a class for testing the charm in the unit test framework. """
    @classmethod
    def setUpClass(cls):
        """ Set up an amulet deployment using the bundle. """
        d = amulet.Deployment()

        local_path = os.path.join(os.path.dirname(__file__), 'local.yaml')
        with open(local_path, "r") as fd:
            config = yaml.safe_load(fd)

        url = config.get('wasnd').get('was_nd_url')
        print('Using URL  %s' % url)
        # Test if a IB url for the repository is defined
        if not url:
            print("You need to define a url for the WAS ND packages repository.\n"
                  "Edit local.yaml or tests/00-setup and run it again.")
            sys.exit(1)

        pkg = config.get('wasnd').get('im_file_name')
        print('Using IBM IM package %s' % pkg)
        if not pkg:
            print("You need to define a package name for IBM IM.\n"
                  "Edit local.yaml or tests/00-setup and run it again.")
            sys.exit(1)

        bundle_path = os.path.join(os.path.dirname(__file__), 'bundles.yaml')
        with open(bundle_path, 'r') as bundle_file:
            contents = yaml.safe_load(bundle_file)
        d.load(contents)
        d.add('haproxy')

        # Software doesn't actually install until you accept the license
        d.configure('was-nd', {'accept-ibm-was-nd-license': True, 'accept-ibm-im-license': True, 'was_nd_url': url, 'im_file_name': pkg })
        d.configure('haproxy', {'services': '- { service_name: was-base-service, service_host: 0.0.0.0, service_port: 9060, service_options: [balance leastconn, cookie SRVNAME insert], server_options: maxconn 100 cookie S check }'})
        d.relate('haproxy:reverseproxy', 'was-nd:website')
        d.expose('haproxy')
        d.setup(seconds_to_wait)
        d.sentry.wait(seconds_to_wait)
        cls.d = d


    def test_deployed(self):
        """ Test to see if the bundle deployed successfully. """
        self.assertTrue(self.d.deployed)

    def test_website(self):
        unit = self.d.sentry.unit['was-nd/0']
        url  = 'http://%s:9060/ibm/console' % unit.info['public-address']
        response = requests.get(url)
        # Raise an exception if the url was not a valid web page.
        response.raise_for_status()

    def test_reverseproxy(self):
        unit = self.d.sentry.unit['haproxy/0']
        haproxy_address = unit.info['public-address']
        haurl  = 'http://%s:9060/ibm/console' % haproxy_address
        page = requests.get(haurl)
        # Raise an exception if the url was not a valid web page.
        page.raise_for_status()
        print('Successfully got the WAS ND web page through haproxy IP address.')

if __name__ == '__main__':
    unittest.main()
